@@ -6,8 +6,6 @@
   See the accompanying LICENSE file.
 */
 
-#ifdef EXPERIMENTAL
-
 /**
 
 .. _backup:
@@ -17,29 +15,29 @@ Backup
 
 A backup object encapsulates copying one database to another.  You
 call :meth:`Connection.backup` on the destination database to get the
-backup object.  Call :meth:`~backup.step` to copy some pages
+Backup object.  Call :meth:`~Backup.step` to copy some pages
 repeatedly dealing with errors as appropriate.  Finally
-:meth:`~backup.finish` cleans up committing or rolling back and
+:meth:`~Backup.finish` cleans up committing or rolling back and
 releasing locks.
 
 Here is an example usage using the **with** statement to ensure
-:meth:`~backup.finish` is called::
+:meth:`~Backup.finish` is called::
 
   # copies source.main into db
   with db.backup("main", source, "main") as b:
       while not b.done:
           b.step(100)
-          print b.remaining, b.pagecount, "\r",
+          print(b.remaining, b.pagecount, "\r", flush = True)
 
 If you are not using **with** then you'll need to ensure
-:meth:`~backup.finish` is called::
+:meth:`~Backup.finish` is called::
 
   # copies source.main into db
   b=db.backup("main", source, "main")
   try:
       while not b.done:
           b.step(100)
-          print b.remaining, b.pagecount, "\r",
+          print(b.remaining, b.pagecount, "\r", flush = True)
   finally:
       b.finish()
 
@@ -64,7 +62,7 @@ The destination database is locked during the copy.  You will get a
     }                                                                                                                      \
   } while (0)
 
-/** .. class:: backup
+/** .. class:: Backup
 
   You create a backup instance by calling :meth:`Connection.backup`.
 */
@@ -92,8 +90,7 @@ APSWBackup_init(APSWBackup *self, Connection *dest, Connection *source, sqlite3_
   self->dest = dest;
   self->source = source;
   self->backup = backup;
-  self->done = Py_False;
-  Py_INCREF(self->done);
+  self->done = Py_NewRef(Py_False);
   self->inuse = 0;
   self->weakreflist = NULL;
 }
@@ -126,7 +123,7 @@ APSWBackup_close_internal(APSWBackup *self, int force)
       PyErr_Fetch(&etype, &eval, &etb);
 
       SET_EXC(res, self->dest->db);
-      apsw_write_unraiseable(NULL);
+      apsw_write_unraisable(NULL);
 
       PyErr_Restore(etype, eval, etb);
       break;
@@ -157,14 +154,14 @@ APSWBackup_dealloc(APSWBackup *self)
 
   Py_CLEAR(self->done);
 
-  Py_TYPE(self)->tp_free((PyObject *)self);
+  Py_TpFree((PyObject *)self);
 }
 
-/** .. method:: step([npages=All]) -> bool
+/** .. method:: step(npages: int = -1) -> bool
 
   Copies *npages* pages from the source to destination database.  The source database is locked during the copy so
   using smaller values allows other access to the source database.  The destination database is always locked until the
-  backup object is :meth:`finished <backup.finish>`.
+  backup object is :meth:`finished <Backup.finish>`.
 
   :param npages: How many pages to copy. If the parameter is omitted
      or negative then all remaining pages are copied. The default page
@@ -176,32 +173,38 @@ APSWBackup_dealloc(APSWBackup *self)
   unable to lock the source database.  You can catch those and try
   again.
 
-  :returns: True if this copied the last remaining outstanding pages, else false.  This is the same value as :attr:`~backup.done`
+  :returns: True if this copied the last remaining outstanding pages, else false.  This is the same value as :attr:`~Backup.done`
 
   -* sqlite3_backup_step
 */
 static PyObject *
-APSWBackup_step(APSWBackup *self, PyObject *args)
+APSWBackup_step(APSWBackup *self, PyObject *args, PyObject *kwds)
 {
-  int pages = -1, res;
+  int npages = -1, res;
 
   CHECK_USE(NULL);
   CHECK_BACKUP_CLOSED(NULL);
 
-  if (args && !PyArg_ParseTuple(args, "|i:step(pages=All)", &pages))
-    return NULL;
+  {
+    static char *kwlist[] = {"npages", NULL};
+    Backup_step_CHECK;
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "|i:" Backup_step_USAGE, kwlist, &npages))
+      return NULL;
+  }
+  PYSQLITE_BACKUP_CALL(res = sqlite3_backup_step(self->backup, npages));
+
+  /* this would happen if there were errors deep in the vfs */
+  MakeExistingException();
 
-  PYSQLITE_BACKUP_CALL(res = sqlite3_backup_step(self->backup, pages));
   if (PyErr_Occurred())
     return NULL;
 
   if (res == SQLITE_DONE)
   {
-    if (self->done != Py_True)
+    if (!Py_IsTrue(self->done))
     {
       Py_CLEAR(self->done);
-      self->done = Py_True;
-      Py_INCREF(self->done);
+      self->done = Py_NewRef(Py_True);
     }
     res = SQLITE_OK;
   }
@@ -212,11 +215,10 @@ APSWBackup_step(APSWBackup *self, PyObject *args)
     return NULL;
   }
 
-  Py_INCREF(self->done);
-  return self->done;
+  return Py_NewRef(self->done);
 }
 
-/** .. method:: finish()
+/** .. method:: finish() -> None
 
   Completes the copy process.  If all pages have been copied then the
   transaction is committed on the destination database, otherwise it
@@ -243,18 +245,18 @@ APSWBackup_finish(APSWBackup *self)
   Py_RETURN_NONE;
 }
 
-/** .. method:: close([force=False])
+/** .. method:: close(force: bool = False) -> None
 
-  Does the same thing as :meth:`~backup.finish`.  This extra api is
+  Does the same thing as :meth:`~Backup.finish`.  This extra api is
   provided to give the same api as other APSW objects such as
-  :meth:`Connection.close`, :meth:`blob.close` and
+  :meth:`Connection.close`, :meth:`Blob.close` and
   :meth:`Cursor.close`.  It is safe to call this method multiple
   times.
 
   :param force: If true then any exceptions are ignored.
 */
 static PyObject *
-APSWBackup_close(APSWBackup *self, PyObject *args)
+APSWBackup_close(APSWBackup *self, PyObject *args, PyObject *kwds)
 {
   int force = 0, setexc;
 
@@ -264,9 +266,13 @@ APSWBackup_close(APSWBackup *self, PyObject *args)
   if (!self->backup)
     Py_RETURN_NONE; /* already closed */
 
-  if (args && !PyArg_ParseTuple(args, "|i:close(force=False)", &force))
-    return NULL;
-
+  {
+    static char *kwlist[] = {"force", NULL};
+    Backup_close_CHECK;
+    argcheck_bool_param force_param = {&force, Backup_close_force_MSG};
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "|O&:" Backup_close_USAGE, kwlist, argcheck_bool, &force_param))
+      return NULL;
+  }
   setexc = APSWBackup_close_internal(self, force);
   if (setexc)
     return NULL;
@@ -274,43 +280,45 @@ APSWBackup_close(APSWBackup *self, PyObject *args)
 }
 
 /** .. attribute:: remaining
+  :type: int
 
   Read only. How many pages were remaining to be copied after the last
-  step.  If you haven't called :meth:`~backup.step` or the backup
-  object has been :meth:`finished <backup.finish>` then zero is
+  step.  If you haven't called :meth:`~Backup.step` or the backup
+  object has been :meth:`finished <Backup.finish>` then zero is
   returned.
 
   -* sqlite3_backup_remaining
 */
 static PyObject *
-APSWBackup_get_remaining(APSWBackup *self, APSW_ARGUNUSED void *ignored)
+APSWBackup_get_remaining(APSWBackup *self, void *Py_UNUSED(ignored))
 {
   CHECK_USE(NULL);
-  return PyInt_FromLong(self->backup ? sqlite3_backup_remaining(self->backup) : 0);
+  return PyLong_FromLong(self->backup ? sqlite3_backup_remaining(self->backup) : 0);
 }
 
 /** .. attribute:: pagecount
+  :type: int
 
   Read only. How many pages were in the source database after the last
-  step.  If you haven't called :meth:`~backup.step` or the backup
-  object has been :meth:`finished <backup.finish>` then zero is
+  step.  If you haven't called :meth:`~Backup.step` or the backup
+  object has been :meth:`finished <Backup.finish>` then zero is
   returned.
 
   -* sqlite3_backup_pagecount
 */
 static PyObject *
-APSWBackup_get_pagecount(APSWBackup *self, APSW_ARGUNUSED void *ignored)
+APSWBackup_get_pagecount(APSWBackup *self, void *Py_UNUSED(ignored))
 {
   CHECK_USE(NULL);
-  return PyInt_FromLong(self->backup ? sqlite3_backup_pagecount(self->backup) : 0);
+  return PyLong_FromLong(self->backup ? sqlite3_backup_pagecount(self->backup) : 0);
 }
 
-/** .. method:: __enter__() -> self
+/** .. method:: __enter__() -> Backup
 
   You can use the backup object as a `context manager
   <http://docs.python.org/reference/datamodel.html#with-statement-context-managers>`_
-  as defined in :pep:`0343`.  The :meth:`~backup.__exit__` method ensures that backup
-  is :meth:`finished <backup.finish>`.
+  as defined in :pep:`0343`.  The :meth:`~Backup.__exit__` method ensures that backup
+  is :meth:`finished <Backup.finish>`.
 */
 static PyObject *
 APSWBackup_enter(APSWBackup *self)
@@ -318,26 +326,27 @@ APSWBackup_enter(APSWBackup *self)
   CHECK_USE(NULL);
   CHECK_BACKUP_CLOSED(NULL);
 
-  Py_INCREF(self);
-  return (PyObject *)self;
+  return Py_NewRef((PyObject *)self);
 }
 
-/** .. method:: __exit__() -> False
+/** .. method:: __exit__(etype: Optional[type[BaseException]], evalue: Optional[BaseException], etraceback: Optional[types.TracebackType]) -> Optional[bool]
 
-  Implements context manager in conjunction with :meth:`~backup.__enter__` ensuring
-  that the copy is :meth:`finished <backup.finish>`.
+  Implements context manager in conjunction with :meth:`~Backup.__enter__` ensuring
+  that the copy is :meth:`finished <Backup.finish>`.
 */
 static PyObject *
-APSWBackup_exit(APSWBackup *self, PyObject *args)
+APSWBackup_exit(APSWBackup *self, PyObject *args, PyObject *kwds)
 {
-  PyObject *etype, *evalue, *etb;
+  PyObject *etype, *evalue, *etraceback;
   int setexc;
 
   CHECK_USE(NULL);
-
-  if (!PyArg_ParseTuple(args, "OOO", &etype, &evalue, &etb))
-    return NULL;
-
+  {
+    static char *kwlist[] = {"etype", "evalue", "etraceback", NULL};
+    Backup_exit_CHECK;
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "OOO:" Backup_exit_USAGE, kwlist, &etype, &evalue, &etraceback))
+      return NULL;
+  }
   /* If already closed then we are fine - CHECK_BACKUP_CLOSED not needed*/
   if (!self->backup)
     Py_RETURN_FALSE;
@@ -347,7 +356,7 @@ APSWBackup_exit(APSWBackup *self, PyObject *args)
      close exception has more detail.  At the time of writing this
      code the step method only set an error code but not an error
      message */
-  setexc = APSWBackup_close_internal(self, etype != Py_None || evalue != Py_None || etb != Py_None);
+  setexc = APSWBackup_close_internal(self, !Py_IsNone(etype) || !Py_IsNone(evalue) || !Py_IsNone(etraceback));
 
   if (setexc)
   {
@@ -359,81 +368,44 @@ APSWBackup_exit(APSWBackup *self, PyObject *args)
 }
 
 /** .. attribute:: done
+  :type: bool
 
-  A boolean that is True if the copy completed in the last call to :meth:`~backup.step`.
+  A boolean that is True if the copy completed in the last call to :meth:`~Backup.step`.
 */
 static PyMemberDef backup_members[] = {
     /* name type offset flags doc */
-    {"done", T_OBJECT, offsetof(APSWBackup, done), READONLY, "True if all pages copied"},
+    {"done", T_OBJECT, offsetof(APSWBackup, done), READONLY, Backup_done_DOC},
     {0, 0, 0, 0, 0}};
 
 static PyGetSetDef backup_getset[] = {
     /* name getter setter doc closure */
-    {"remaining", (getter)APSWBackup_get_remaining, NULL, "Pages still to be copied", NULL},
-    {"pagecount", (getter)APSWBackup_get_pagecount, NULL, "Total pages in source database", NULL},
+    {"remaining", (getter)APSWBackup_get_remaining, NULL, Backup_remaining_DOC, NULL},
+    {"pagecount", (getter)APSWBackup_get_pagecount, NULL, Backup_pagecount_DOC, NULL},
     {0, 0, 0, 0, 0}};
 
 static PyMethodDef backup_methods[] = {
     {"__enter__", (PyCFunction)APSWBackup_enter, METH_NOARGS,
-     "Context manager entry"},
-    {"__exit__", (PyCFunction)APSWBackup_exit, METH_VARARGS,
-     "Context manager exit"},
-    {"step", (PyCFunction)APSWBackup_step, METH_VARARGS,
-     "Copies some pages"},
+     Backup_enter_DOC},
+    {"__exit__", (PyCFunction)APSWBackup_exit, METH_VARARGS | METH_KEYWORDS,
+     Backup_exit_DOC},
+    {"step", (PyCFunction)APSWBackup_step, METH_VARARGS | METH_KEYWORDS,
+     Backup_step_DOC},
     {"finish", (PyCFunction)APSWBackup_finish, METH_NOARGS,
-     "Commits or rollsback backup"},
-    {"close", (PyCFunction)APSWBackup_close, METH_VARARGS,
-     "Alternate way to finish"},
+     Backup_finish_DOC},
+    {"close", (PyCFunction)APSWBackup_close, METH_VARARGS | METH_KEYWORDS,
+     Backup_close_DOC},
     {0, 0, 0, 0}};
 
 static PyTypeObject APSWBackupType =
     {
-        APSW_PYTYPE_INIT
-        "apsw.backup",                                                          /*tp_name*/
-        sizeof(APSWBackup),                                                     /*tp_basicsize*/
-        0,                                                                      /*tp_itemsize*/
-        (destructor)APSWBackup_dealloc,                                         /*tp_dealloc*/
-        0,                                                                      /*tp_print*/
-        0,                                                                      /*tp_getattr*/
-        0,                                                                      /*tp_setattr*/
-        0,                                                                      /*tp_compare*/
-        0,                                                                      /*tp_repr*/
-        0,                                                                      /*tp_as_number*/
-        0,                                                                      /*tp_as_sequence*/
-        0,                                                                      /*tp_as_mapping*/
-        0,                                                                      /*tp_hash */
-        0,                                                                      /*tp_call*/
-        0,                                                                      /*tp_str*/
-        0,                                                                      /*tp_getattro*/
-        0,                                                                      /*tp_setattro*/
-        0,                                                                      /*tp_as_buffer*/
-        Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_VERSION_TAG, /*tp_flags*/
-        "backup object",                                                        /* tp_doc */
-        0,                                                                      /* tp_traverse */
-        0,                                                                      /* tp_clear */
-        0,                                                                      /* tp_richcompare */
-        offsetof(APSWBackup, weakreflist),                                      /* tp_weaklistoffset */
-        0,                                                                      /* tp_iter */
-        0,                                                                      /* tp_iternext */
-        backup_methods,                                                         /* tp_methods */
-        backup_members,                                                         /* tp_members */
-        backup_getset,                                                          /* tp_getset */
-        0,                                                                      /* tp_base */
-        0,                                                                      /* tp_dict */
-        0,                                                                      /* tp_descr_get */
-        0,                                                                      /* tp_descr_set */
-        0,                                                                      /* tp_dictoffset */
-        0,                                                                      /* tp_init */
-        0,                                                                      /* tp_alloc */
-        0,                                                                      /* tp_new */
-        0,                                                                      /* tp_free */
-        0,                                                                      /* tp_is_gc */
-        0,                                                                      /* tp_bases */
-        0,                                                                      /* tp_mro */
-        0,                                                                      /* tp_cache */
-        0,                                                                      /* tp_subclasses */
-        0,                                                                      /* tp_weaklist */
-        0                                                                       /* tp_del */
-        APSW_PYTYPE_VERSION};
-
-#endif /* EXPERIMENTAL */
+        PyVarObject_HEAD_INIT(NULL, 0)
+            .tp_name = "apsw.Backup",
+        .tp_basicsize = sizeof(APSWBackup),
+        .tp_dealloc = (destructor)APSWBackup_dealloc,
+        .tp_flags = Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,
+        .tp_doc = Backup_class_DOC,
+        .tp_weaklistoffset = offsetof(APSWBackup, weakreflist),
+        .tp_methods = backup_methods,
+        .tp_members = backup_members,
+        .tp_getset = backup_getset,
+};
